@@ -30,7 +30,7 @@ pub struct Cli {
     #[structopt(short = "w", long, value_name = "FLOAT", default_value = "0")]
     pub max_qual: f32,
 
-    /// Verbose output statistics [multiple up to -vvv]  
+    /// Verbose output statistics [multiple, up to -vvv]  
     #[structopt(
         short,
         long,
@@ -38,14 +38,26 @@ pub struct Cli {
     )]
     pub verbose: u64,
 
+    /// Header for summary output
+    #[structopt(short = "H", long)]
+    pub header: bool,
+
     /// Number of top reads in verbose summary.  
     #[structopt(short, long, value_name = "INT", default_value = "5")]
     pub top: usize,
 
-    /// Summary statistics report
+    /// Summary report only [stdout].
     #[structopt(short, long)]
     pub stats: bool,
 
+    /// Summary report output file.
+    #[structopt(short, long)]
+    pub report: Option<PathBuf>,
+
+    /// Summary report in JSON format.
+    #[structopt(short, long)]
+    pub json: bool,
+
     /// Ignore quality values if present.
     #[structopt(short, long)]
     pub fast: bool,
